# -*- coding: utf-8 -*-
from __future__ import print_function
from ._version import get_versions

__author__ = 'Suyog Soti'
__email__ = 'suyog.soti@gmail.com'
__version__ = get_versions()['version']
del get_versions
